@@ -1,98 +1,9 @@
 import ast
+import getpass
 import os
 import re
 import sys
-import getpass
-
-def cli_input(msg,**opts):
-    hidden=opts.get('hidden',False)
-    if hidden:
-        if sys.stdin.isatty():
-            p = getpass.getpass(msg)
-        else:
-            printf(msg,end_line='')
-            p = sys.stdin.readline().rstrip()
-    else:
-        if sys.version_info[0] == '2':
-            p=raw_input(msg)
-        else:
-            p=input(msg)
-    return p
-
-def IsNone(inp,**opts):
-    default=opts.get('default',opts.get('out',True))
-    if inp is None: 
-       return default
-    elif isinstance(inp,(list,tuple,dict,str)):
-       if not inp:
-           return default
-       if opts.get('detail'):
-           if isinstance(inp,(list,tuple)) and len(inp) == 1:
-               if inp[0] is None:
-                   return default
-               if isinstance(inp[0],(list,tuple,dict,str)) and not inp[0]:
-                   return default
-    if isinstance(default,bool):
-        return False
-    return inp
-
-def space(n=1):
-    s=''
-    for i in range(0,n):
-        s+=' '
-    return s
-
-def tap_string(string,fspace=0,nspace=0,new_line='\n',flength=0,nlength=0,ntap=0):
-    if isinstance(string,str):
-        rc_str=[]
-        string_a=string.split(new_line)
-        n=space(nspace)
-        for ii in range(0,len(string_a)):
-            if ii == 0:
-                s=space(fspace)
-                if flength > 0:
-                    for jj in range(0,len(string_a[ii])//flength):
-                        rc_str.append(s+string_a[ii][flength*jj:(flength*(jj+1))])
-                        if ntap:
-                            s=space(nspace)+space(ntap)
-                            ntap=0
-                    rc_str.append(s+string_a[ii][(flength*(jj+1)):])
-                else:
-                    rc_str.append(s+string_a[ii])
-            else:
-                if nlength > 0:
-                    for jj in range(0,len(string_a[ii])//nlength):
-                        rc_str.append(n+string_a[ii][nlength*jj:(nlength*(jj+1))])
-                        if ntap:
-                            n=n+space(ntap)
-                            ntap=0
-                    rc_str.append(n+string_a[ii][(nlength*(jj+1)):])
-                else:
-                    rc_str.append(n+string_a[ii])
-        return new_line.join(rc_str)
-    return space(fspace)+'{}'.format(string)
-
-def TypeData(_type,data,spliter=None):
-    if _type is str:
-        return '{}'.format(data)
-    elif _type is int:
-        try:
-            return int(data)
-        except:
-            return False
-    elif _type in [list,dict,tuple]:
-        if isinstance(data,str):
-            try:
-                data=ast.literal_eval(data)
-            except:
-                if spliter:
-                    if _type is list : return data.split(spliter)
-                    if _type is tuple : return tuple(data.split(spliter))
-                return False
-    if isinstance(data,_type):
-        return data
-    return False
-
+import kmisc as km
 
 class SysArg:
     '''
@@ -119,26 +30,27 @@ defind()
     def __init__(self,*args,**opts):
         if not args:
             self.argv=tuple(sys.argv[:])
-            self.args=sys.argv
+            self.args=sys.argv[:]
         else:
             self.argv=tuple(args[:])
-            self.args=args
-        self.program=opts.get('program') 
-        self.cmd_id=opts.get('cmd_id',0) 
-        self.desc=opts.get('desc') 
-        self.epilog=opts.get('epilog') 
+            self.args=list(args[:])
+        self.program=opts.get('program')
+        self.cmd_id=opts.get('cmd_id',0)
+        if len(self.args) > self.cmd_id:
+            del self.args[self.cmd_id]
+        self.desc=opts.get('desc')
+        self.epilog=opts.get('epilog')
         self.option={}
         self.group={}
-        self.commands=[]
+        self.commands={}
         self.version=opts.get('version')
-        self.help_desc=opts.get('help_desc','Help') 
-        self.ask=opts.get('ask',False) 
-
+        self.help_desc=opts.get('help_desc','Help')
+        self.ask=opts.get('ask',False)
+        self.SysArg_hidden_show=opts.get('SysArg_hidden_show','SysArg_hidden_show')
     def error_exit(self,msg):
         sys.stderr.write('{}\n'.format(msg))
         sys.stderr.flush()
         os._exit(1)
-
     def define(self,name=None,**opts):
         if name in self.option:
             self.error_exit('Already defined "{}" parameter name'.format(name))
@@ -159,17 +71,19 @@ defind()
         _select=opts.get('select',[])
         _spliter=opts.get('spliter',None) #list or tuple case but input is string
         _arg=opts.get('arg',False)
-        if IsNone(name) and  not _command and not _arg:
+        _cmd_id=opts.get('cmd_id',1)
+        _combin=opts.get('combin',False)
+        _hidden=opts.get('hidden',False)
+        if km.IsNone(name) and  not _command and not _arg:
             if _short:
                 self.error_exit('Required parameter name for option at {}'.format(_short))
             else:
                 self.error_exit('Required parameter name for option at {}'.format(_long))
-
         _value=[]
         # location parameter(value)
         if not _short and not _long and len(self.argv) > _params:
             _value=self.argv[_params]
-            if _value in self.args: self.args.remove(_value)
+            if not _command and _value in self.args: self.args.remove(_value)
         else:
             # Check same option
             for cc in self.option:
@@ -178,14 +92,13 @@ defind()
                 if _long and self.option[cc].get('long') == _long:
                     self.error_exit('ERROR: Already "{} of {}" defined at {}'.format(_long,name,cc))
             # Check same option in each group
-            if _group in self.group: 
+            if _group in self.group:
                 for cc in self.group[_group]:
                     if not isinstance(self.group[_group][cc],dict): continue
                     if _short and self.group[_group][cc].get('short') == _short:
                         self.error_exit('ERROR: Already "{} of {}" defined at {} in {}'.format(_short,name,cc,_group))
                     if _long and self.group[_group][cc].get('long') == _long:
                         self.error_exit('ERROR: Already "{} of {}" defined at {} in {}'.format(_long,name,cc,_group))
-
             # Check parameter
             if _params == 0:
                 if _long in self.args:
@@ -195,13 +108,13 @@ defind()
                     if _short in self.args:
                         _value=True
                         self.args.remove(_short)
-#                    else:
-#                        for ii in range(0,len(self.args)):
-#                            if re.match(r'-[a-zA-Z0-9]',self.args[ii]):
-#                                if _short[1:] in self.args[ii]:
-#                                    _value=True
-#                                    i=self.args[ii].index(_short[1:])
-#                                    self.args[ii]=self.args[ii][:i]+self.args[ii][i+len(_short[1:]):]                
+                    elif _combin: # check combin data in short
+                        for ii in range(0,len(self.args)):
+                            if re.match(r'-[a-zA-Z0-9]',self.args[ii]):
+                                if _short[1:] in self.args[ii]:
+                                    _value=True
+                                    i=self.args[ii].index(_short[1:])
+                                    self.args[ii]=self.args[ii][:i]+self.args[ii][i+len(_short[1:]):]
             # Input parameter
             else:
                 if _long and _params_name:
@@ -209,17 +122,17 @@ defind()
                     tt=[]
                     for ii in self.args:
                         if ii.startswith('{}='.format(_long)):
-                            __v__=TypeData(_type,ii.split('=')[1],_spliter)
+                            __v__=km.TypeData(_type,ii.split('=')[1],_spliter)
                             if __v__ is False:
                                 if self.ask:
-                                    iii=cli_input('Wrong input type format at {}, it required {}! Please type it:'.format(_short if _short else _long,_type.__name__))
-                                    if not IsNone(iii):
-                                        __v__=TypeData(_type,iii,_spliter)
+                                    iii=km.cli_input('Wrong input type format at {}, it required {}! Please type it:'.format(_short if _short else _long,_type.__name__))
+                                    if not km.IsNone(iii):
+                                        __v__=km.TypeData(_type,iii,_spliter)
                                         if __v__ is False:
-                                            self.error_exit('ERROR: Wrong input type format at {}, it required {}'.format(_short if _short else _long,_type.__name__)) 
+                                            self.error_exit('ERROR: Wrong input type format at {}, it required {}'.format(_short if _short else _long,_type.__name__))
                                         _value.append(__v__)
                                 else:
-                                    self.error_exit('ERROR: Wrong input type format at {}, it required {}'.format(_long,_type.__name___)) 
+                                    self.error_exit('ERROR: Wrong input type format at {}, it required {}'.format(_long,_type.__name___))
                             _value.append(__v__)
                         else:
                             tt.append(ii)
@@ -232,17 +145,17 @@ defind()
                                 for jj in range(ii+1,len(self.args)):
                                     if self.args[jj][0] == '-':
                                         break
-                                    __v__=TypeData(_type,self.args[jj],_spliter)
+                                    __v__=km.TypeData(_type,self.args[jj],_spliter)
                                     if __v__ is False:
                                         if self.ask:
-                                            iii=cli_input('Wrong input type format at {}, it required {}! Please type it:'.format(_short if _short else _long,_type.__name__))
-                                            if not IsNone(iii):
-                                                __v__=TypeData(_type,iii,_spliter)
+                                            iii=km.cli_input('Wrong input type format at {}, it required {}! Please type it:'.format(_short if _short else _long,_type.__name__))
+                                            if not km.IsNone(iii):
+                                                __v__=km.TypeData(_type,iii,_spliter)
                                                 if __v__ is False:
-                                                    self.error_exit('ERROR: Wrong input type format at {}, it required {}'.format(_short if _short else _long,_type.__name__)) 
+                                                    self.error_exit('ERROR: Wrong input type format at {}, it required {}'.format(_short if _short else _long,_type.__name__))
                                                 _value.append(__v__)
                                         else:
-                                            self.error_exit('ERROR: Wrong input type format at {}, it required {}'.format(_short if _short else _long,_type.__name__)) 
+                                            self.error_exit('ERROR: Wrong input type format at {}, it required {}'.format(_short if _short else _long,_type.__name__))
                                     _value.append(__v__)
                                 break
                                 tt=tt+self.args[jj+1:]
@@ -250,17 +163,17 @@ defind()
                                 for jj in range(0,_params):
                                     if self.args[ii+1+jj][0] == '-':
                                         break
-                                    __v__=TypeData(_type,self.args[ii+1+jj],_spliter)
+                                    __v__=km.TypeData(_type,self.args[ii+1+jj],_spliter)
                                     if __v__ is False:
                                         if self.ask:
-                                            iii=cli_input('Wrong input type format at {}, it required {}! Please type it:'.format(_short if _short else _long,_type.__name__))
-                                            if not IsNone(iii):
-                                                __v__=TypeData(_type,iii,_spliter)
+                                            iii=km.cli_input('Wrong input type format at {}, it required {}! Please type it:'.format(_short if _short else _long,_type.__name__))
+                                            if not km.IsNone(iii):
+                                                __v__=km.TypeData(_type,iii,_spliter)
                                                 if __v__ is False:
-                                                    self.error_exit('ERROR: Wrong input type format at {}, it required {}'.format(_short if _short else _long,_type.__name__)) 
+                                                    self.error_exit('ERROR: Wrong input type format at {}, it required {}'.format(_short if _short else _long,_type.__name__))
                                                 _value.append(__v__)
                                         else:
-                                            self.error_exit('ERROR: Wrong input type format at {}, it required {}'.format(_short if _short else _long,_type.__name__)) 
+                                            self.error_exit('ERROR: Wrong input type format at {}, it required {}'.format(_short if _short else _long,_type.__name__))
                                     _value.append(__v__)
                                 if len(_value) == _params:
                                     tt=tt+self.args[ii+jj+2:]
@@ -271,13 +184,13 @@ defind()
                         else:
                             tt.append(self.args[ii])
                     self.args=tt
-            
         if _group:
             if _command:
-                if _group not in self.commands: self.commands.append(_group)
+                if _group not in self.commands:self.commands[_group]=_cmd_id
             if _group not in self.group: self.group[_group]={}
             if _command:
                 self.group[_group]['command']=_command
+                if _hidden: self.group[_group]['hidden']=_hidden
                 #If command has _select and arg then this command should be need input arg value in selection
                 if _arg:
                     self.group[_group]['arg']=True
@@ -306,6 +219,8 @@ defind()
                     'required':_required,
                     'spliter':_spliter,
                     'select':_select,
+                    'combin':_combin,
+                    'hidden':_hidden,
                 }
             if _group_desc: self.group[_group]['desc']=_group_desc
         else:
@@ -321,30 +236,44 @@ defind()
                 'required':_required,
                 'spliter':_spliter,
                 'select':_select,
+                'combin':_combin,
+                'hidden':_hidden,
             }
-
-    def Cmd(self,name=None):
-        if self.cmd_id == 0 : return self.argv[0]
-        if len(self.argv) > self.cmd_id:
-            if self.argv[self.cmd_id] in self.commands:
-                if name:
-                    if self.argv[self.cmd_id] == name:
-                        self.Check(group=name)
-                        return True
-                    return False
-                self.Check(group=self.argv[self.cmd_id])
-                return self.argv[self.cmd_id]
-            if self.argv[self.cmd_id] not in ['--version','--help']:
-                print(':: Wrong command "{}"\n'.format(self.argv[self.cmd_id]))
+    def Cmd(self,name=None,cmd_group=False):
+        sys_argvn=len(sys.argv)
+        # name is current right command?
+        if name:
+            if name in self.commands:
+                if sys_argvn > self.commands[name] and sys.argv[self.commands[name]] == name:
+                    self.Check(group=name)
+                    if len(self.args) > self.commands[name] and self.args[self.commands[name]-1] == name: del self.args[self.commands[name]-1]
+                    return True
+            else:
+                if sys_argvn > self.cmd_id and sys.argv[self.cmd_id] == name:
+                    del self.args[self.cmd_id]
+                    return True
+            return False
+        # What is currently my command?
+        for i in self.commands:
+            if sys_argvn > self.commands[i] and sys.argv[self.commands[i]] == i:
+                if len(self.args) > self.commands[i] and self.args[self.commands[i]-1] == i: del self.args[self.commands[i]-1]
+                self.Check(group=i)
+                return i
+        if cmd_group:
+            print('! Can not find right command!\n')
         else:
-            print(':: Require some command, check cmd_id=N in SysArg()\n')
+            if self.cmd_id < sys_argvn:
+                self.Check(group=self.argv[self.cmd_id])
+                if len(self.args) > self.cmd_id and self.args[self.cmd_id] == self.argv[self.cmd_id]: del self.args[self.cmd_id]
+                return self.argv[self.cmd_id]
+            else:
+                print(':: Require some command, check cmd_id=N in SysArg()\n')
         self.Help(call=True)
-
     def Get(self,name=None,group=None,mode='auto'):
         def Val(data,mode='auto'):
             if isinstance(data,dict):
                 vv=data.get('value')
-                if IsNone(vv):
+                if km.IsNone(vv):
                     vv=data.get('default')
                 if data.get('select'):
                     if vv in data.get('select'):
@@ -355,60 +284,56 @@ defind()
                     if isinstance(vv,list) and len(vv) == 1:
                         return vv[0]
                 return vv
-
         if group and group in self.group:
             if name:
-                return IsNone(Val(self.group[group][name]),out=None)
-            return IsNone(Val(self.group[group]),out=None)
+                return Val(self.group[group][name])
+            return Val(self.group[group])
         elif name in self.option:
-            return IsNone(Val(self.option[name]),out=None)
+            return Val(self.option[name])
         else:
             rt={}
             if not name and not group:
                 for o in self.option:
-                    rt[o]=IsNone(Val(self.option[o]),out=None)
+                    rt[o]=Val(self.option[o])
                 for g in self.group:
                     if g not in rt: rt[g]={}
                     for o in self.group[g]:
                         if isinstance(self.group[g][o],dict):
-                            rt[g][o]=IsNone(Val(self.group[g][o]),out=None)
+                            rt[g][o]=Val(self.group[g][o])
             elif not name and group and group in self.group:
                 rt[group]={}
                 for o in self.group[group]:
                     if isinstance(self.group[group][o],dict):
-                        rt[group][o]=IsNone(Val(self.group[group][o]),out=None)
-            return IsNone(rt,out=None)
-
-
+                        rt[group][o]=km.IsNone(Val(self.group[group][o]),out=None)
+            return rt
     def Check(self,group=None):
         if group and group in self.group:
             for name in self.group[group]:
                 if isinstance(self.group[group][name],dict) and self.group[group][name].get('required'):
-                    if IsNone(self.group[group][name].get('value')):
-                        if IsNone(self.group[group][name].get('default')):
+                    if km.IsNone(self.group[group][name].get('value')):
+                        if km.IsNone(self.group[group][name].get('default')):
                             aa=self.group[group][name].get('long') if self.group[group][name].get('long') else self.group[group][name].get('short')
                             if self.ask:
-                                iii=cli_input('Missing "{}({})" parameter! Please type it:'.format(aa,name))
-                                if not IsNone(iii):
+                                iii=km.cli_input('Missing "{}({})" parameter! Please type it:'.format(aa,name))
+                                if not km.IsNone(iii):
                                     self.group[group][name]['value']=iii
                             else:
                                 sys.stdout.write('\n!! Missing required option "{}({})" of {} !!\n\n'.format(aa,name,group))
                                 sys.stdout.flush()
-                                self.Help(call=True)
+                                self.Help(call=True,command=group)
         for name in self.option:
             if self.option[name].get('required'):
-                if IsNone(self.option[name].get('value')):
-                    if IsNone(self.option[name].get('default')):
+                if km.IsNone(self.option[name].get('value')):
+                    if km.IsNone(self.option[name].get('default')):
                         aa=self.option[name].get('long') if self.option[name].get('long') else self.option[name].get('short')
                         if self.ask:
-                            iii=cli_input('Missing "{}({})" parameter! Please type it:'.format(aa,name))
-                            if not IsNone(iii):
+                            iii=km.cli_input('Missing "{}({})" parameter! Please type it:'.format(aa,name))
+                            if not km.IsNone(iii):
                                 self.option[name]['value']=iii
                         else:
                             sys.stdout.write('\n!! Missing required option "{}({})" !!\n\n'.format(name,aa))
                             sys.stdout.flush()
                             self.Help(call=True)
-        
     def Version(self,version=None,call=False,new_line='\n'):
         if (version or self.version) and '--version' in sys.argv:
             if version:
@@ -420,8 +345,7 @@ defind()
             if new_line: sys.stdout.write(new_line)
             sys.stdout.flush()
             os._exit(0)
-
-    def Help(self,Short='-h',Long='--help',call=False,short_len=5,long_len=30,desc_space=2):
+    def Help(self,Short='-h',Long='--help',call=False,short_len=5,long_len=30,desc_space=2,command=None,ignore_unknown_command=False):
        #######################
        #Description Design
        #######################
@@ -451,7 +375,6 @@ defind()
                    else '(V,V,..)' if _type is tuple \
                    else "{'S':'V',..}" if _type is dict \
                    else 'S'
-                       
                if isinstance(_params,int) and _params:
                    if _params==1:
                        _desc=_desc+'(input({}):{})'.format(_params,s)
@@ -471,15 +394,16 @@ defind()
                            _desc=_desc+'(input:{0})'.format(s)
                        else:
                            _desc=_desc+'(input:{0}1 {0}2 ...)'.format(s)
-           if _desc: 
-               return tap_string(_desc,nspace=nspace)
+           if _desc:
+               return km.WrapString(_desc,nspace=nspace)
            return ''
-
        #######################
        #Option Design
        #######################
        def print_option(data):
+           force=km.Var(self.SysArg_hidden_show,False)
            if isinstance(data,dict):
+               if not force and data.get('hidden'): return
                _desc=mk_desc(data.get('desc'),default=data.get('default'),required=data.get('required'),nspace=short_len+long_len+desc_space,_type=data.get('type'),_params=data.get('params'),_params_name=data.get('params_name'),_spliter=data.get('spliter'),_select=data.get('select'))
                if data.get('short') and data.get('long'):
                    if len(data.get('short')) > short_len:
@@ -496,47 +420,93 @@ defind()
                        sss=long_len-(len(data.get('short'))-short_len)
                    else:
                        sss=long_len
-                   sys.stdout.write('%{}s  %s%s\n'.format(short_len)%(data.get('short'),space(sss),_desc))
+                   sys.stdout.write('%{}s  %s%s\n'.format(short_len)%(data.get('short'),km.Space(sss),_desc))
                    sys.stdout.flush()
                elif data.get('long'):
                    if data.get('params_name'):
-                       sys.stdout.write('%s  %-{}s%s\n'.format(long_len)%(space(short_len),'{}={}'.format(data.get('long'),data.get('params_name')),_desc))
+                       sys.stdout.write('%s  %-{}s%s\n'.format(long_len)%(km.Space(short_len),'{}={}'.format(data.get('long'),data.get('params_name')),_desc))
                    else:
-                       sys.stdout.write('%s  %-{}s%s\n'.format(long_len)%(space(short_len),data.get('long'),_desc))
+                       sys.stdout.write('%s  %-{}s%s\n'.format(long_len)%(km.Space(short_len),data.get('long'),_desc))
                    sys.stdout.flush()
-
-
+       # If exist command then print help for the command only (show all)
+       if Short in self.args:
+           ii=self.args.index(Short)
+           if ii > 0:
+               command=self.args[ii-1]
+       if Long in self.args:
+           ii=self.args.index(Long)
+           if ii> 0:
+               command=self.args[ii-1]
+       if command:
+           if command in self.group:
+               #if command but no option then ignore
+               if len(self.group[command]) >= 2:
+                   #Command group
+                   if self.group[command].get('command'):
+                       print()
+                       sys.stdout.write('Usage: {} {} [OPTION] [<args>]\n'.format(self.program,command))
+                       print()
+                       if self.group[command].get('desc'):
+                           _group_desc=km.WrapString(self.group[command]['desc'],nspace=short_len+long_len+desc_space)
+                           sys.stdout.write(' %s\n'%(_group_desc))
+                       sys.stdout.write('\n[OPTION]\n')
+                       #Print regular/global option
+                       for oo in self.option:
+                           print_option(self.option[oo])
+                   #normal group
+                   else:
+                       if self.group[command].get('desc'):
+                           sys.stdout.write('%-{}s%s\n'.format(short_len+long_len)%(' * {}'.format(command),_group_desc))
+                       else:
+                           sys.stdout.write('%-{}s\n'.format(short_len+long_len)%(' * {}'.format(command)))
+                   sys.stdout.flush()
+                   #Print group/local option
+                   for oo in self.group[command]:
+                       print_option(self.group[command][oo])
+                   #Print other group option
+                   if self.group[command].get('command'):
+                       for gg in self.group:
+                           if not self.group[gg].get('command'):
+                               if self.group[gg].get('desc'):
+                                   _group_desc=km.WrapString(self.group[gg]['desc'],nspace=short_len+long_len+desc_space)
+                                   sys.stdout.write('\n%-{}s%s\n'.format(short_len+long_len)%(' * {}'.format(gg),_group_desc))
+                               else:
+                                   sys.stdout.write('\n%-{}s\n'.format(short_len+long_len)%(' * {}'.format(gg)))
+                               for oo in self.group[gg]:
+                                   print_option(self.group[gg][oo])
+                   os._exit(0)
+           elif not ignore_unknown_command:
+               print('{} not found'.format(command))
+               os._exit(1)
        #######################
        #Print Help(Main Design)
        #######################
        #if call or (Short and Short in self.argv) or (Long and Long in self.argv):
-       if call or (Short and Short in sys.argv) or (Long and Long in sys.argv):
+       if call or km.IsSame(km.Get(self.args,0),Short)  or km.IsSame(km.Get(self.args,0),Long):
            if self.cmd_id > 0 and not self.commands:
                 sys.stderr.write("ERROR: defined extra command at SysArg(..,cmd_id=N).\nIt required defineing command with SysArg.define(...,group=<cmd name>,command=True)\n")
                 sys.stderr.flush()
-                os._exit(1) 
-
+                os._exit(1)
            #Print Program
            if self.program:
                if self.option:
                    if self.commands:
-                       sys.stdout.write('Usage: {} <command> [OPTION] [<args>]\n'.format(self.program))
+                       sys.stdout.write('\nUsage: {} <command> [OPTION] [<args>]\n'.format(self.program))
                    else:
-                       sys.stdout.write('Usage: {} [OPTION] [<args>]\n'.format(self.program))
+                       sys.stdout.write('\nUsage: {} [OPTION] [<args>]\n'.format(self.program))
                else:
                    if self.commands:
-                       sys.stdout.write('Usage: {} <command> [<args>]\n'.format(self.program))
+                       sys.stdout.write('\nUsage: {} <command> [<args>]\n'.format(self.program))
                    else:
-                       sys.stdout.write('Usage: {} [<args>]\n'.format(self.program))
+                       sys.stdout.write('\nUsage: {} [<args>]\n'.format(self.program))
                if self.version:
-                   sys.stdout.write('Version: {}\n'.format(self.version))
+                   sys.stdout.write('Version: {}\n\n'.format(self.version))
                sys.stdout.flush()
-
            #Print Special Group Option
            if self.commands and self.cmd_id > 0 and len(self.argv) > self.cmd_id and self.argv[self.cmd_id] in self.commands:
                if self.group[self.argv[self.cmd_id]].get('command'):
                    if self.group[self.argv[self.cmd_id]].get('desc'):
-                       _group_desc=tap_string(self.group[self.argv[self.cmd_id]]['desc'],nspace=short_len+long_len+desc_space)
+                       _group_desc=km.WrapString(self.group[self.argv[self.cmd_id]]['desc'],nspace=short_len+long_len+desc_space)
                        sys.stdout.write('* %-{}s  %s\n'.format(short_len+long_len-2)%(self.argv[self.cmd_id],_group_desc))
                    else:
                        sys.stdout.write('* %-{}s\n'.format(short_len+long_len-2)%(self.argv[self.cmd_id]))
@@ -544,18 +514,18 @@ defind()
                for oo in self.group[self.argv[self.cmd_id]]:
                    print_option(self.group[self.argv[self.cmd_id]][oo])
                os._exit(0)
-
            #Print Desc
            if self.desc:
-               sys.stdout.write(self.desc)
+               sys.stdout.write(self.desc+'\n')
                sys.stdout.flush()
-
-           #Commands Help
+           force=km.Var(self.SysArg_hidden_show,False)
+           #Supported Commands display Description
            if self.commands:
                sys.stdout.write('\nSupported <command>s are:\n')
                for cc in self.commands:
-                   if self.group.get(cc,{}).get('desc'):
-                       _group_desc=tap_string(self.group[cc]['desc'],nspace=short_len+long_len+desc_space)
+                   if not force and self.group.get(cc,{}).get('hidden'): continue
+                   if self.group.get(cc,{}).get('desc') :
+                       _group_desc=km.WrapString(self.group[cc]['desc'],nspace=short_len+long_len+desc_space)
                        if self.group.get(cc,{}).get('arg'): # required argument
                            sys.stdout.write('  %-{}s%s\n'.format(short_len+long_len)%('{} [OPT] <arg>'.format(cc),_group_desc))
                        else:
@@ -564,43 +534,56 @@ defind()
                        sys.stdout.write('  %-{}s\n'.format(short_len+long_len)%(cc))
                    sys.stdout.flush()
                sys.stdout.flush()
-
            sys.stdout.write('\n[OPTION]\n')
            #Print Help Option
-           _help_desc=tap_string(self.help_desc,nspace=short_len+long_len+desc_space)
+           _help_desc=km.WrapString(self.help_desc,nspace=short_len+long_len+desc_space)
            if Short and Long:
                sys.stdout.write('%{}s, %-{}s%s\n'.format(short_len,long_len)%(Short,Long,_help_desc))
            elif Long:
-               sys.stdout.write('%s  %-{}s%s\n'.format(long_len)%(space(short_len),Long,_help_desc))
+               sys.stdout.write('%s  %-{}s%s\n'.format(long_len)%(km.Space(short_len),Long,_help_desc))
            elif Short and Long:
-               sys.stdout.write('%{}s  %s\n'.format(short_len)%(Short,space(long_len),_help_desc))
+               sys.stdout.write('%{}s  %s\n'.format(short_len)%(Short,km.Space(long_len),_help_desc))
            sys.stdout.flush()
-            
            #Print Regular Option
            for oo in self.option:
                print_option(self.option[oo])
-
            #Print Group Option
            for gg in self.group:
-               #if command but no option then ignore
-               if len(self.group[gg]) > 3:
+               #if command but no option/hidden then ignore
+               group_hidden=self.group[gg].pop('hidden') if 'hidden' in self.group[gg] else None
+               if len(self.group[gg]) >= 3 and (not group_hidden or force):
                    print()
                    if self.group[gg].get('desc'):
-                       _group_desc=tap_string(self.group[gg]['desc'],nspace=short_len+long_len+desc_space)
+                       _group_desc=km.WrapString(self.group[gg]['desc'],nspace=short_len+long_len+desc_space)
                        sys.stdout.write('%-{}s%s\n'.format(short_len+long_len)%(' * {}'.format(gg),_group_desc))
                    else:
                        sys.stdout.write('%-{}s\n'.format(short_len+long_len)%(' * {}'.format(gg)))
                    sys.stdout.flush()
                    for oo in self.group[gg]:
                        print_option(self.group[gg][oo])
-
            #Print Epilog
            if self.epilog:
                print(self.epilog)
            os._exit(0)
-            
     def Args(self):
         return self.__dict__.get('args')
-
-
+    def Opts(self,name=None,get_true=False,combin=False):
+        out={}
+        o=self.__dict__.get('option',{})
+        for i in o:
+            if o[i].get('value'):
+                if get_true and o[i].get('value') != get_true: continue
+                out[o[i].get('short',o[i].get('long'))]=o[i].get('value')
+        o=self.__dict__.get('group',{}).get(name,{})
+        for i in o:
+            if not isinstance(o[i],dict): continue
+            if o[i].get('value'):
+                if get_true and o[i].get('value') != get_true: continue
+                out[o[i].get('short',o[i].get('long'))]=o[i].get('value')
+        if combin:
+            m=''
+            for i in out.keys():
+                m=m+i[1:]
+            return '-{}'.format(m)
+        return out
 
